@@ -3,7 +3,7 @@
 rpqueue (Redis Priority Queue)
 
 Originally written July 5, 2011
-Copyright 2011-2013 Josiah Carlson
+Copyright 2011-2015 Josiah Carlson
 Released under the GNU LGPL v2.1
 available: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 
@@ -11,6 +11,7 @@ Other licenses may be available upon request.
 '''
 
 import datetime
+from hashlib import sha1
 import itertools
 try:
     import simplejson as json
@@ -35,6 +36,9 @@ except ImportError:
 
 import redis
 
+if map(int, redis.__version__.split('.')) < [2, 4, 12]:
+    raise Exception("Upgrade your Redis client to version 2.4.12 or later")
+
 QUEUES_KNOWN = 'queues:all'
 QUEUES_PRIORITY = 'queues:wall'
 QUEUE_KEY = 'queue:'
@@ -56,37 +60,41 @@ QUEUE_ITEMS_PER_PASS = 100
 
 REDIS_CONNECTION_SETTINGS = {}
 POOL = None
-PID = None
 
+LOG_LEVELS = dict((v, getattr(logging, v)) for v in ['DEBUG', 'INFO', 'WARNING', 'ERROR'])
+LOG_LEVEL = 'debug'
 logging.basicConfig()
 log_handler = logging.root
 
+SUCCESS_LOG_LEVEL = 'debug'
+AFTER_FORK = None
+
 def _assert(condition, message, *args):
     if not condition:
         raise ValueError(message%args)
 
 def script_load(script):
     '''
-    Borrowed from my book, Redis in Action:
+    Borrowed and updated from my book, Redis in Action:
     https://github.com/josiahcarlson/redis-in-action/blob/master/python/ch11_listing_source.py
     '''
-    sha = [None]
+    sha = [None, sha1(script).hexdigest()]
     def call(conn, keys=[], args=[], force_eval=False):
         if not force_eval:
             if not sha[0]:
-                sha[0] = conn.execute_command(
-                    "SCRIPT", "LOAD", script, parse="LOAD")
+                try:
+                    return conn.execute_command("EVAL", script, len(keys), *(keys+args))
+                finally:
+                    del sha[:-1]
 
             try:
-                return conn.execute_command(
-                    "EVALSHA", sha[0], len(keys), *(keys+args))
+                return conn.execute_command("EVALSHA", sha[0], len(keys), *(keys+args))
 
             except redis.exceptions.ResponseError as msg:
                 if not msg.args[0].startswith("NOSCRIPT"):
                     raise
 
-        return conn.execute_command(
-            "EVAL", script, len(keys), *(keys+args))
+        return conn.execute_command("EVAL", script, len(keys), *(keys+args))
 
     return call
 
@@ -100,17 +108,22 @@ def set_redis_connection_settings(host='localhost', port=6379, db=0,
     REDIS_CONNECTION_SETTINGS = locals()
     if redis.__version__ < '2.4':
         REDIS_CONNECTION_SETTINGS.pop('unix_socket_path', None)
-    POOL = threading.local()
+    POOL = None
+
+def set_redis_connection(conn):
+    '''
+    Sets the global pooled connection to the provided connection object. Useful
+    for environments where additional pooling or other options are desired or
+    required.
+    '''
+    global POOL
+    POOL = conn
 
 def get_connection():
-    global PID
-    if PID != os.getpid():
-        # handle fork() after connection, and initial connection
-        PID = os.getpid()
-        set_redis_connection_settings(**REDIS_CONNECTION_SETTINGS)
-    if not getattr(POOL, 'conn', None):
-        POOL.conn = redis.Redis(**REDIS_CONNECTION_SETTINGS)
-    return POOL.conn
+    global POOL
+    if not POOL:
+        POOL = redis.Redis(**REDIS_CONNECTION_SETTINGS)
+    return POOL
 
 def _enqueue_call(conn, queue, fname, args, kwargs, delay=0, taskid=None):
     '''
@@ -270,7 +283,6 @@ def handle_delayed_python_(conn, queues, timeout):
     to_end = time.time() + timeout
     first = True
     found = False
-    qm1 = len(queues) - 1
     i = 0
     while not SHOULD_QUIT[0] and (time.time() <= to_end or first):
         # remove old locks every pass
@@ -662,6 +674,8 @@ PREVIOUS = None
 def quit_on_signal(signum, frame):
     SHOULD_QUIT[0] = 1
 
+SUCCESS_LOG = None
+
 def execute_tasks(queues=None, threads_per_process=1, processes=1, wait_per_thread=1, module=None):
     '''
     Will execute tasks from the (optionally) provided queues until the first
@@ -671,7 +685,7 @@ def execute_tasks(queues=None, threads_per_process=1, processes=1, wait_per_thre
     EXECUTE_TASKS = True
     signal.signal(signal.SIGUSR1, quit_on_signal)
     signal.signal(signal.SIGTERM, quit_on_signal)
-    log_handler.setLevel(logging.DEBUG)
+    log_handler.setLevel(LOG_LEVELS[LOG_LEVEL.upper()])
     sp = []
     def _signal_subprocesses(signum, frame):
         for pp in sp:
@@ -716,6 +730,11 @@ def execute_task_threads(queues=None, threads=1, wait_per_thread=1, module=None)
     signal.signal(signal.SIGUSR2, _print_stackframes_on_signal)
     if module:
         __import__(module)
+    if AFTER_FORK:
+        try:
+            AFTER_FORK()
+        except:
+            log_handler.exception("ERROR: Exception in AFTER_FORK function: %s", traceback.format_exc().rstrip())
     st = []
     log_handler.info("PID %i executing tasks in %i threads", os.getpid(), threads)
     for t in xrange(threads-1):
@@ -747,6 +766,10 @@ def _execute_task(work, conn):
         log_handler.exception(err)
         return
 
+    global SUCCESS_LOG
+    if SUCCESS_LOG is None:
+        SUCCESS_LOG = getattr(log_handler, SUCCESS_LOG_LEVEL.lower(), log_handler.debug)
+
     now = datetime.datetime.utcnow()
     jitter = time.time() - scheduled
     log_handler.debug("RECEIVED: %s %s at %r (%r late)", taskid, fname, now, jitter)
@@ -764,7 +787,7 @@ def _execute_task(work, conn):
     except:
         log_handler.exception("ERROR: Exception in task %r: %s", to_execute, traceback.format_exc().rstrip())
     else:
-        log_handler.debug("SUCCESS: Task completed: %s %s", taskid, fname)
+        SUCCESS_LOG("SUCCESS: Task completed: %s %s", taskid, fname)
 
 def set_priority(queue, qpri, conn=None):
     '''
